      SUBROUTINE TRANS_LENGTH_ESSL(N_WANT, N_GET )
C***BEGIN PROLOGUE                                     TRANS_LENGTH_ESSL
C                                                      *****************
C***PURPOSE  Find the length, N_GET, of a DFT which is .GE. N_WANT for
C            which the ESSL DFT routines can compute a DFT
C***AUTHOR   Robert E. Lynch, Purdue Univ., West Lafayette, IN, 2000
C***DESCRIPTION

C *Usage:
C     INTEGER N_WANT, N_GET
C
C     CALL TRANS_LENGTH_ESSL( N_WANT, N_GET )

C *Arguments:
C    On Entry:
C        INTEGER N_WANT:  The desired length of a (1D) DFT transform
C
C    On Return:
C        INTEGER N_GET:   Length of a (1D) DFT transform which ESSL can
C                         compute. N_WANT is equal to
C                            (2**I2)*(3**I3)*(5**I5)*(7**I7)*(11**I11)
C                         with 1 <= I2 <=25,  0 <= I3 <= 2,
C                              0 <= I5,I7,I11 <= 1
C                         If N_WANT has this form, then N_GET is set
C                         equal to N_WANT.  Otherwise, N_GET is set
C                         equal to the next larger integer which does
C                         have this form.
C *Errors:
C    If N_WANT is not between  2  and  4620, which are the first and
C    last entries in the array NTABLE, then a message is printed,
C    ERROR_STOP is called, and execution terminates.  ESSL can
C    compute transforms with lengths up to 36700160.  One can increase
C    the number of entries in NTABLE to include a larger set of values
C    (see p. 742 of the 1997 edition of the ESSL manual); one could
C    also have ESSL compute an acceptible value provided one makes
C    certain error `recoverable' (see the manual).
C 
C *Method
C    Binary search in NTABLE
C 
C***ROUTINES CALLED (NONE; except, possibly, ERROR_STOP)
C***INCLUDE FILES   allprogs.inc
C***REVISION HISTORY (YYMMDD)
C   201201 Date initially written
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C***END PROLOGUE
      IMPLICIT NONE
      INCLUDE '../include/allprogs.inc'
C Declare Arguments
      INTEGER  N_WANT, N_GET
C Delcare Local Variables
      INTEGER N, N_LOW, N_HIGH, V, V_LOW, V_HIGH, V_WANT, KOUNT
      INTEGER NUMB_TRANS_LENGTHS 
      PARAMETER ( NUMB_TRANS_LENGTHS =  140 )
      INTEGER NTABLE( NUMB_TRANS_LENGTHS )
      INTEGER J

C     NTABLE contains the first NUMB_TRANS_LENGTHS transform lengths
C     which ESSL can use as a (1d) DFT length.
      DATA (NTABLE(J),J=1,NUMB_TRANS_LENGTHS)/
     +     2,    4,    6,    8,   10,   12,   14,   16,   18,   20,
     +    22,   24,   28,   30,   32,   36,   40,   42,   44,   48,
     +    56,   60,   64,   66,   70,   72,   80,   84,   88,   90,
     +    96,  110,  112,  120,  126,  128,  132,  140,  144,  154,
     +   160,  168,  176,  180,  192,  198,  210,  220,  224,  240,
     +   252,  256,  264,  280,  288,  308,  320,  330,  336,  352,
     +   360,  384,  396,  420,  440,  448,  462,  480,  504,  512,
     +   528,  560,  576,  616,  630,  640,  660,  672,  704,  720, 
     +   768,  770,  792,  840,  880,  896,  924,  960,  990, 1008,
     +  1024, 1056, 1120, 1152, 1232, 1260, 1280, 1320, 1344, 1386,
     +  1408, 1440, 1536, 1540, 1584, 1680, 1760, 1792, 1848, 1920,
     +  1980, 2016, 2048, 2112, 2240, 2304, 2310, 2464, 2520, 2560,
     +  2640, 2688, 2772, 2816, 2880, 3072, 3080, 3168, 3360, 3520,
     +  3584, 3696, 3840, 3960, 4032, 4096, 4224, 4480, 4608, 4620 /

      CHARACTER*16   SUBNAME
      DATA SUBNAME / 'SET_TRANS_LENGTH' /
 
C     Use bisection method to find entry in NTABLE
      V_WANT = N_WANT
      N_LOW  = 1
      V_LOW  = NTABLE(N_LOW)
      N_HIGH = NUMB_TRANS_LENGTHS
      V_HIGH = NTABLE(N_HIGH)
      IF( ( N_WANT .LT. V_LOW ) .OR. ( V_HIGH .LT. N_WANT ) ) THEN
        PRINT 1030, N_WANT, V_LOW, V_HIGH, V_HIGH
 1030   FORMAT(' Requested transform length = ',i5/
     +         ' Acceptible values are between ', I5,' and ',I5/
     +         ' If values larger than ',I5,' are needed, then ',
     +         ' augment the array NTABLE')
        CALL ERROR_STOP(PROGNAME, SUBNAME, 
     +    ' Size of desired transform length is out of range.')
      ENDIF
 
      DO KOUNT = 1, NUMB_TRANS_LENGTHS
        N = INT( ( N_LOW + N_HIGH ) / 2 )
        V = NTABLE(N)

        IF( V .EQ. V_WANT ) THEN
          N_GET = V
                                                              GO TO 1000
C                                                             **********
        ELSEIF( V .LT. V_WANT ) THEN
          N_LOW = N
        ELSEIF( V_WANT .LT. V ) THEN
          N_HIGH = N
        ENDIF

        IF( N_HIGH - N_LOW .EQ. 1 ) THEN
          IF( NTABLE(N_LOW) .EQ. V_WANT ) THEN
            N_GET = V_WANT
          ELSE
            N_GET = NTABLE(N_HIGH)
          ENDIF
                                                              GO TO 1000
C                                                             **********
        ENDIF

      ENDDO
 1000 CONTINUE

      RETURN
      END
