C *****************************************************************
C *                                                               *
C * Subroutine: output_density                                    *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   Apr.20, 1998                                          *
C * Used in: aamain -- The main routine.                          *
C * Function: Write densitys to output map file.                  *
C * Usage:                                                        *
C *   call output_density (density, nx, ny, nz)                   *
C * Input:                                                        *
C *   density (Real) -- The density to be written into external   *
C *                     file.                                     *
C *   nx (Integer) -- The limits of the size of array "density"   *
C *                   on X direction.                             *
C *   ny (Integer) -- The limits of the size of array "density"   *
C *                   on Y direction.                             *
C *   nz (Integer) -- The limits of the size of array "density"   *
C *                   on Z direction.                             *
C *                                                               *
C * Algorithm:                                                    *
C *   "density" is the array to be rotated and written; "nx",     *
C *   "ny", "nz" are coordinates of X, Y, and Z axies,            *
C *   corresponding column, row, and section of Fortran array.    *
C *                                                               *
C *   UNIT 8 = OUTPUT FILE CONTAINING 3D MAP. Ouput filename is   *
C *   defined in "info.inc" and read in subroutine "info".        *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002      *
C *     - R Sinkovits, general cleanup, 8/25/06                   *

C     Note that this routine as currently written has two deficiencies
C     (1) All writes go through root process
C     (2) The original array density_map on root process is overwritten

      SUBROUTINE output_density (DENSITY_MAP, 
     +     den_min_x, den_max_x, max_z_slab, MAP_MIN, MAP_MAX)

      IMPLICIT NONE

      include 'mpif.h'
      INCLUDE '../include/allprogs.inc'
      INCLUDE '../include/info.inc'
      INCLUDE '../include/pifhead.inc'
      
      type (PIFHEAD_T) MAPHEAD, IMGHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD

      integer, intent(in) :: den_max_x, den_min_x, max_z_slab
      REAL, intent(in)    :: MAP_MIN, MAP_MAX

      real, intent(inout) ::   density_map( -den_min_x+den_max_x+1,
     +     -den_min_x+den_max_x+1, max_z_slab)
      
      INTEGER SEC, f_z, l_z, nx, ny, nz
      integer i,j
      integer stat(mpi_status_size)
      REAL    SCALE1, MAP_SCALEF
      character(len=1) :: versionStr(11), READWRITE(4)
      real    zerosec(den_min_x:den_max_x, den_min_x:den_max_x)

      integer*2 i2
      integer*4 i4

      character*14 subname
      data subname / 'output_density' /
      data versionStr(1) / 'l' /, versionStr(2) / 'i' /,
     +       versionStr(3) / 'b' /, versionStr(4) / 'P' /,
     +       versionStr(5) / 'I' /, versionStr(6) / 'F' /,
     +       versionStr(7) / ' ' /, versionStr(8) / '2' /,
     +       versionStr(9) / '.' /, versionStr(10) / '1' /,
     +       versionStr(11) / '6' /

      zerosec = 0.0 ! f90 array syntax

      f_z = max(frst_lst_z(1,IAM), den_min_x)
      l_z = min(frst_lst_z(2,IAM), den_max_x)

      nx = den_max_x * 2 +1
      ny = nx
      nz = nx

C     ----- Write header -----
      if (iam == 0) then
         
         CALL PIF_INIT_HEAD (MAPHEAD)
         MAPHEAD%LUNUM = 8
         READWRITE(1)  = 'w'
         CALL PIF_OPEN (out_file_name, MAPHEAD%LUNUM, READWRITE)
         IF(pixhd_htyp == 1) THEN
            MAPHEAD%NUMCOLS = ny
            MAPHEAD%NUMROWS = nx
            MAPHEAD%NUMSECS = nz
         ELSE
            call error_stop (progname, subname, 'Undefined pixhd_htype')
         ENDIF
         
         MAPHEAD%NIMG       =    1
         MAPHEAD%IMGNUM     =    1
         SCALE1             = 1.01*MAX(ABS(MAP_MIN),ABS(MAP_MAX))
         MAP_SCALEF         = real(huge(i4)) / SCALE1
         MAPHEAD%int4scaleF = MAP_SCALEF
         if (outmode == 40) then
            MAPHEAD%MODE     =   40
         elseif (outmode == 20) then
            MAPHEAD%MODE     =   20
            MAP_SCALEF       = real(huge(i2)) / SCALE1
         else
            call error_stop (progname, subname, 'Undefined output mode')
         endif
         MAPHEAD%SCALEF     = MAP_SCALEF
         MAPHEAD%recipFloatScaleFactor = 1.0 / MAP_SCALEF
         MAPHEAD%endianNess = 0
         DO I =1,11
            MAPHEAD%versionStr(i) = versionStr(i)
         ENDDO
         MAPHEAD%img_bkg    = 0
         MAPHEAD%BINFACTOR  = pixhd_htyp
         MAPHEAD%A_CELL     = ny
         MAPHEAD%B_CELL     = nx
         MAPHEAD%C_CELL     = nz
         MAPHEAD%dataI4_min = MAP_MIN
         MAPHEAD%dataI4_max = MAP_MAX
         MAPHEAD%DATAR4_MIN = 0
         MAPHEAD%DATAR4_MAX = 0
         MAPHEAD%pixelSize  = PIXSIZ_OUT / mapmagfactor
         
         CALL PIF_WRITE_GH (MAPHEAD)
         MAPHEAD%IMGNUM = 1
         CALL PIF_WRITE_DH (MAPHEAD)
      endif
      
C     ----- Write density map -----
      if(iam == 0) then
         SEC = 1

C     Write out the leftmost sections that are not on process 0
         if (f_z <= l_z .and. f_z > den_min_x) then
            do i = den_min_x, f_z - 1
               if (outmode == 40) then
                  call PIF_WRITE_SECI4(MAPHEAD,zerosec,nx,ny,SEC)
               elseif (outmode == 20) then
                  call PIF_WRITE_SECI2(MAPHEAD,zerosec,nx,ny,SEC)
               endif
               sec = sec +1
            enddo
         endif

C     Write out root process's share of density map
         DO i = f_z , l_z 
            if (outmode == 40) then
               CALL PIF_WRITE_SECI4 (MAPHEAD,
     +              density_map(1,1,i-frst_lst_z(1,IAM)+1),nx,ny,SEC)
            elseif (outmode == 20) then
               CALL PIF_WRITE_SECI2 (MAPHEAD,
     +              density_map(1,1,i-frst_lst_z(1,IAM)+1),nx,ny,SEC)
            endif
            sec = sec +1
         ENDDO
         
C     Get contributions from non-root processes and write to file
         DO i = 1, num_nodes-1
            f_z = max( frst_lst_z(1,i), den_min_x)
            l_z = min( frst_lst_z(2,i), den_max_x)
            if ( f_z <= l_z ) then
               call MPI_RECV( density_map(1,1,1),
     +              (l_z - f_z + 1)* ((-den_min_x+den_max_x+1) **2),
     +              mpi_real, i, i, mpi_comm_world, stat, mpi_err) 
               DO j = 1, l_z - f_z + 1
                  if (outmode == 40) then
                     CALL PIF_WRITE_SECI4 (MAPHEAD,
     +                    density_map(1,1,j),nx,ny,SEC)
                  elseif (outmode == 20) then
                     CALL PIF_WRITE_SECI2 (MAPHEAD,
     +                    density_map(1,1,j),nx,ny,SEC)
                  endif
                  sec = sec +1
               ENDDO 
            endif
         ENDDO

C     Write out the rightmost sections that are not on process n-1
         if (l_z < den_max_x)  then
            do i = l_z + 1, den_max_x
               if (outmode == 40) then
                  call PIF_WRITE_SECI4(MAPHEAD,zerosec,nx,ny,SEC)
               elseif (outmode == 20) then
                  call PIF_WRITE_SECI2(MAPHEAD,zerosec,nx,ny,SEC)
               endif
               sec = sec +1
            enddo
         endif
         
         CALL PIF_CLOSE (MAPHEAD%LUNUM)
         if (sec .NE. nz+1) call error_stop(progname, subname, 
     +        'write map is not a cube')
         
      else 

C     Send chunks of density map to root process
         if (f_z <= l_z) then
            call MPI_SEND( density_map(1,1,f_z - frst_lst_z(1,IAM)+1),
     +           (l_z - f_z + 1) * ((-den_min_x+den_max_x+1) **2),
     +           mpi_real, 0, iam, mpi_comm_world, mpi_err )
            if (mpi_err.ne.0) call error_stop(progname, subname,
     +           'Terminated because of  mpi_send  error')
         endif

      endif

      return
      end subroutine output_density
