      subroutine ico_vector(theta, phi, ico_x, ico_y, ico_z)

C     For a given orientation (theta, phi) specified in units of
C     degrees, generate the full set of unit vectors that are related by
C     icosahedral symmetry

      implicit none

      real, intent(in)  :: theta, phi
      real, intent(out) :: ico_x(60), ico_y(60), ico_z(60)
      
      real theta_rad, phi_rad, deg_to_rad, cos36, cos60, cos72
      integer i

      deg_to_rad = acos(-1.0)/180.0
      theta_rad = theta * deg_to_rad
      phi_rad = phi * deg_to_rad

      ico_x(1) = sin(theta_rad) * cos(phi_rad)
      ico_y(1) = sin(theta_rad) * sin(phi_rad)
      ico_z(1) = cos(theta_rad)
      
      cos36 = cos(36.0*deg_to_rad)
      cos60 = cos(60.0*deg_to_rad)
      cos72 = cos(72.0*deg_to_rad)
      
C     Apply 5-fold symmetry.operations
      do i = 1,4
         ico_x(i+1) =  cos36*ico_x(i) + cos72*ico_y(i) + cos60*ico_z(i)
         ico_y(i+1) =  cos72*ico_x(i) + cos60*ico_y(i) - cos36*ico_z(i)
         ico_z(i+1) = -cos60*ico_x(i) + cos36*ico_y(i) + cos72*ico_z(i)
      enddo

C     Apply 3-fold symmetry operations
      do i = 1,5
         ico_x(i+5)  = ico_z(i)
         ico_y(i+5)  = ico_x(i)
         ico_z(i+5)  = ico_y(i)
         ico_x(i+10) = ico_y(i)
         ico_y(i+10) = ico_z(i)
         ico_z(i+10) = ico_x(i)
      enddo

C     Apply 2-fold symmetry operations
      do i = 1,15
         ico_x(i+15) =  ico_x(i)
         ico_y(i+15) = -ico_y(i)
         ico_z(i+15) = -ico_z(i)
      enddo

      do i = 1,30
         ico_x(i+30) = -ico_x(i)
         ico_y(i+30) =  ico_y(i)
         ico_z(i+30) = -ico_z(i)
      enddo

      return
      end subroutine ico_vector
