       subroutine ctf_para(FFT,abs_ctf,IDIM1, IDIM2, CTFMODE, FILTER,
     +     TEMPFAC, CTF_FF1, CTF_FF2, PIXSIZE, UNITS, VOLT, 
     +     FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, Weight,
     +     res_max, res_min)

C  *********************************************************************
Cji generate the ctf factor and ctf amplitude weight
C  * THE FORMULA USED depends on the value of FILTER:
C  *
C  * FILTER FORMULA ( FILT_FAC is CTF_FF1)
C ----------------------------------------------------------------------
C  *   1    FILTER 4 (before first peak)+ FILTER 2 (after first peak)  *
C  *   2    FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 +
C  *                              FILT_FAC*(1-ABS(CTF))
C  *   3    FFTnew = (FFTold*ABS(CTF)*exp(TFAC))/(CTF**2 + FILT_FAC)   *
C  *   4    FFTnew = (FFTold*exp(TFAC))/(ABS(CTF)+FILT_FAC*(1-ABS(CTF))*
C  *   5    FFTnew = (FFTold*exp(TFAC))/(ABS(CTF)+FILT_FAC)
C  *
C  * FILT_FAC = FACTOR TO MAKE SURE NOISE IS NOT MAGNIFIED             *
C  *            TOO MUCH NEAR THE NODES OF THE CTF.
C  * TFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS
C  *            USED TO SHARPEN UP HIGHER RESOUTION FEATURES.  VALUES
C  *            FOR CRYOTEM DATA TYPICALLY RANGE BETWEEN 100-500 A**2. *
C  *            THIS TEMPERATURE FACTOR IS APPLIED AS AN INVERSE T_FAC *
C  *
C  * CTFMODE = 0 DO NOTHING!
C  *         = 1 CORRECT PHASES AND AMPLITUDES
C  *         = 2 FLIP PHASES WHERE NECESSARY;LEAVE AMPLITUDES UNCHANGED*
C  *         = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED
C  *         = 4 ONLY apply temperature factor
C  *********************************************************************
C
C *   - Yongchang Ji,  Aug.2000 -- Jan. 2002                           *
Cji   - Yongchang Ji, April,23,2003 add multi Filter to ctf            *
Cji        NOTE: some dummy valuse are used, there are must be
C                wrongs with original CTF from Bio of Purdue
C                     

        IMPLICIT NONE
C
Cji parameters
        INTEGER*4  CTFMODE, IDIM1, IDIM2, UNITS, FILTER
        REAL*4     FFT(0:IDIM1, -IDIM2:IDIM2), 
     +           abs_ctf(0:IDIM1, -IDIM2:IDIM2),
     +           PIXSIZE, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     +           AMP_FAC, Cs, CTF_FF1, CTF_FF2, TEMPFAC, VOLT, Weight,
     +           res_max, res_min


Cji local
        INTEGER*4   IX, IY, idim
        REAL*4      XSQ, CTF, D_STAR, PSI, PHI, C, T
        REAL*4      FOCUS, PI, RADIAN, WAVELENGTH, X, Y, TMP
        REAL*4      Cc, BETA, dE, FACTOR, FFT_RAD, FFT_RAD_TEST
        REAL*4      ds_max, ds_min, t_max, delsq, minimum
        LOGICAL   ASTIG

C ----------------------
        IF(CTFMODE.LE.0 .OR. CTFMODE.GT.4) RETURN
C
        Cc   = 0.0                    ! These are dummy values 
        BETA = 0.0
        dE   = 0.0
        idim = IDIM1*2+1
        minimum = 0.000001
C
Cji     WAVELENGTH  = 1.23 / SQRT (VOLT + (1.0E-6 * VOLT**2))
        WAVELENGTH =1.2264264967/sqrt(VOLT+ 0.9784663543E-6 * VOLT**2)

        PI     = 4.0*ATAN2(1.0,1.0)
        RADIAN = PI/180.0
        PHI = ANG_MAJ * RADIAN          ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION         
        IF(UNITS.EQ.2) THEN
           TMP = 1.0/(IDIM*PIXSIZE)                    ! ALREADY IN NM UNITS
         ELSE
           TMP = 10.0/(IDIM*PIXSIZE)                   ! CONVERT TO NM UNITS
        ENDIF
        IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
           ASTIG = .FALSE.
           FOCUS = FOCUS_MAJ
         ELSE
           ASTIG = .TRUE.
        ENDIF
C
Cji add for softborder
C
        ds_max = (idim*pixsize) / res_max
        ds_min = (idim*pixsize) / res_min
        t_max = exp((TEMPFAC/400.0) * (ds_min*tmp)**2)
        delsq = -(ds_max-ds_min)**2 / log(minimum/t_max)
C
        DO IX = -IDIM2, IDIM2                      ! PROCESS FFT A column AT A TIME
          X = REAL(IX)
          XSQ = X**2
          DO IY = 0, IDIM1                          ! PROCESS CURRENT column 
            Y = REAL(IY)
            D_STAR = SQRT(XSQ + Y**2) * TMP
            IF(ABS(Y).GT.0.9) THEN              ! TRAP FOR FFT Y-AXIS DATA
               PSI = ATAN2 (X,Y)                ! ATAN2 GIVES ANGLE IN RADIANS
             ELSE
               IF(X.GE.0.0) THEN
                  PSI =  PI/2.0
                ELSE
                  PSI = -PI/2.0
               ENDIF
            ENDIF
C
            IF (ASTIG) 
     +          CALL CTF_ASTIG (FOCUS,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
            C = CTF (D_STAR, PI,WAVELENGTH, FOCUS, AMP_FAC,
     1               Cs, Cc, BETA, dE, VOLT)
            abs_ctf(IY,IX)  = abs(C)
            T = EXP ((TEMPFAC/400.0) * (D_STAR**2))             ! ADJUSTED FOR NM
            FFT_RAD_TEST = D_STAR / TMP
C 
Cji-----------*  APPLY CORRECT FILTER    *
C
            IF (FILTER.EQ.1) THEN
               CALL CTF_FIRSTPEAK (FOCUS, WAVELENGTH, FFT_RAD,
     1                            PIXSIZE, IDIM, Cs, units)
               IF (FFT_RAD_TEST .LE. FFT_RAD) THEN
                  FACTOR = T /(ABS(C) + CTF_FF1*(1.0-ABS(C)))
               ELSE
                  FACTOR = (T*abs(C))/(C*C + CTF_FF2*(1.0-abs(C)))
                  if (abs(C) .lt. 0.1) then !!! Changed from sqrt(ctf_ff2)
                     FACTOR = 0.0
                     abs_ctf(IY,IX) = 0.0
                  endif
               ENDIF
            ELSEIF(FILTER.EQ.2) THEN
               FACTOR = (T*ABS(C))/(C*C + CTF_FF1*(1.0-ABS(C))) ! "Misha+Wei"
               if (abs(C).lt.sqrt(CTF_FF1)) then
                 FACTOR = 0.0
                 abs_ctf(IY,IX) = 0.0
               endif
            ELSEIF(FILTER.EQ.3) THEN
               FACTOR = (T*ABS(C))/(C*C + CTF_FF1)
            ELSEIF(FILTER.EQ.4) THEN
               FACTOR = T/(ABS(C) + CTF_FF1*(1.0-ABS(C)))
            ELSEIF(FILTER.EQ.5) THEN
               FACTOR = T/(ABS(C) + CTF_FF1)
            ENDIF
C
Cji         if ctfmode = 4, we only apply temper factor 
            IF (CTFMODE.EQ.4) FACTOR = T
C
Cji add for softborder
C
            if (res_max .lt. res_min) then                        ! means ds_max > ds_min
               if (fft_rad_test .gt. ds_min .and. 
     +             fft_rad_test .le. ds_max)    then
                   FACTOR = FACTOR * t_max / T *
     +                      exp(-(fft_rad_test-ds_min)**2/delsq)
               elseif (fft_rad_test.gt.ds_max) then
                   FACTOR = 0.
               endif
            endif
C
! Old ctf factor that we used before      
Cji            FACTOR = T/(ABS(C) + CTF_FF1*(1.0-abs(c))) 
C
Cji-----------*  APPLY CORRECT CTFMODE    *
C
            IF(CTFMODE.EQ.1) THEN                ! ADJUST PHASES AND AMPLITUDES
               IF(C.GT.0.0) FACTOR = -FACTOR
               FFT(IY,IX)   =   Weight * FACTOR
            ELSEIF(CTFMODE.EQ.2) THEN            ! ADJUST PHASES ONLY
               FACTOR = 1
               IF(C.GT.0.0) FACTOR = -1
               FFT(IY,IX)   =   Weight * FACTOR
            ELSEIF(CTFMODE.EQ.3) THEN            ! ADJUST AMPLITUDES ONLY
               FFT(IY,IX)   =   Weight * FACTOR
            ELSEIF(CTFMODE.EQ.4) THEN            ! ADJUST TEMPER FACTOR ONLY
               FFT(IY,IX)   =   Weight * FACTOR
            ENDIF
          ENDDO
        ENDDO
C
        RETURN
        END
Cji
C  *********************************************************************
Cji generate the ctf values for the 2D array
C                     
       subroutine ctf_temp(ctf3,IDIM1, IDIM2, PIXSIZE, VOLT,
     +     FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC)

        IMPLICIT NONE
Cji parameters
        INTEGER*4   IDIM1, IDIM2
        REAL*4      ctf3(0:IDIM1, -IDIM2:IDIM2), 
     +            PIXSIZE, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     +            AMP_FAC, Cs, VOLT, TEMPFAC


Cji local
        INTEGER*4   IX, IY, idim
        REAL*4      XSQ, CTF, D_STAR, PSI, PHI
        REAL*4      FOCUS, PI, RADIAN, WAVELENGTH, X, Y, TMP
        REAL*4      Cc, BETA, dE, T
        LOGICAL   ASTIG

C ----------------------
        Cc   = 0.0                    ! These are dummy values 
        BETA = 0.0
        dE   = 0.0
        idim = IDIM1*2+1
C
Cji     WAVELENGTH  = 1.23 / SQRT (VOLT + (1.0E-6 * VOLT**2))
        WAVELENGTH =1.2264264967/sqrt(VOLT+ 0.9784663543E-6 * VOLT**2)

        PI     = 4.0*ATAN2(1.0,1.0)
        RADIAN = PI/180.0
        PHI = ANG_MAJ * RADIAN          ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION         
        TMP = 10.0/(IDIM*PIXSIZE)                   ! CONVERT TO NM UNITS
        IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
           ASTIG = .FALSE.
           FOCUS = FOCUS_MAJ
         ELSE
           ASTIG = .TRUE.
        ENDIF
C
        DO IX = -IDIM2, IDIM2                      ! PROCESS CURRENT ROW
          X = REAL(IX)
          XSQ = X**2
          DO IY = 0, IDIM1                          ! PROCESS FFT A ROW AT A TIME
            Y = REAL(IY)
            D_STAR = SQRT(XSQ + Y**2) * TMP
            IF(ABS(Y).GT.0.9) THEN              ! TRAP FOR FFT Y-AXIS DATA
               PSI = ATAN2 (X,Y)                ! ATAN2 GIVES ANGLE IN RADIANS
             ELSE
               IF(X.GE.0.0) THEN
                  PSI =  PI/2.0
                ELSE
                  PSI = -PI/2.0
               ENDIF
            ENDIF
C
            IF (ASTIG) 
     +          CALL CTF_ASTIG (FOCUS,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
            ctf3(IY,IX) = CTF (D_STAR, PI,WAVELENGTH, FOCUS, AMP_FAC,
     1               Cs, Cc, BETA, dE, VOLT)
C
            T = EXP ((TEMPFAC/400.0) * (D_STAR**2))             ! ADJUSTED FOR NM
C
            ctf3(IY,IX) = abs(ctf3(IY,IX) / T)
          ENDDO
        ENDDO
C
        RETURN
        END
Cji
C  ****************************************************************************
C  *                                                                          *
C  *                          *****     CTF.F     *****                       *
C  *                                                                          *
C  *    LIBRARY OF MISCELLANEOUS CTF SUBROUTINES/FUNCTIONS:                   *
C  *                                                                          *
C  *     CTF          - Compute value of CTF at particular spatial frequency. *
C  *     CTF_ASTIG    - Compute defocus value for astigmatic data.            *
C  *     CTF_Es       - Compute exponential decay from spatial coherence.     *
C  *     CTF_Et       - Compute exponential decay from temporal coherence.    *
C  ****************************************************************************
C
C
C  ****************************************************************************
C  *                                                                          *
C  *                      *****     FUNCTION CTF     *****                    *
C  *                                                                          *
C  *    COMPUTE VALUE OF CTF AT PARTICULAR SPATIAL FREQUENCY, TAKING INTO     *
C  *     ACCOUNT THE DECAY EFFECTS OF SPATIAL AND TEMPORAL COHERENCE.         *
C  *                                                                          *
C  *    The basic formula used here is as described by Bottcher et al.        *
C  *     (Structure 4:387-394[1996]) which is:                                *
C  *                                                                          *
C  *       CTF = -[sqrt(1 - amp_fac**2)*sin(chi) +  amp_fac*cos(chi)]         *
C  *                                                                          *
C  *    To account for spatial and temporal coherence the CTF is then         *
C  *      multiplied by two separate exponential decay functions, Es and Et:  *
C  *                                                                          *
C  *                            CTF = CTF * Es * Et                           *
C  *                                                                          *
C  *    The definitions for Es and Et are found in the appropriate FUNCTION   *
C  *      routines, CTF_Es and CTF_Et.                                        *
C  *                                                                          *
C  *    Descriptins of the variables used to compute CTF are given in the     *
C  *     FUNCTION routines, CTF, CTF_Es, and CTF_Et.                          *
C  *                                                                          *
C  *    Calculations are performed after variables are converted to nm units. *
C  *       AMP_FAC is the fractional amplitude contrast.                      *
C  *       Cs is the spherical aberration coefficient (in mm)                 *
C  *       Cc is the chromatic aberration coefficient (in mm)                 *
C  *       dF is the defocus (in microns; positive for underfocus)            *
C  *       WAVELENGTH is the electron wavelength (in nm)                      *
C  *       D_STAR is the spatial frequency (in nm-1)                          *
C  *       BETA is the illumination aperture (in milliradians)                *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED  1-SEP-1999 BY TSB.                                     * 
C  *    LAST MODIFIED  1-SEP-1999 BY TSB.                                     * 
C  ****************************************************************************
C
	FUNCTION CTF (D_STAR, PI, WAVELENGTH, dF, AMP_FAC, Cs, Cc, BETA, 
     1 dE, VOLTS)
C
	IMPLICIT NONE
C
	REAL*4   CTF, PI, WAVELENGTH, Cs, dF, D_STAR, AMP_FAC
	REAL*4   BETA, Cc, CTF_Es, Es, CTF_Et, Et, dE, VOLTS
	REAL*8 D_CTF, D_STAR_SQ, CHI, PART1, PART2, D_D_STAR
C  ----------------------------------------------------------------------------
C
	Es        = 1.0			! Initialize spatial coherence decay
	Et        = 1.0			! Initialize temporal coherence decay
	D_D_STAR  = D_STAR
	D_STAR_SQ = D_D_STAR * D_D_STAR
	PART1     = ((dF*1000.) * D_STAR_SQ) / 2.0
	PART2     = ((Cs*1000000.) * WAVELENGTH**2 * 
     1             D_STAR_SQ*D_STAR_SQ) / 4.0
	CHI       = 2.0 * PI * WAVELENGTH * (PART1 - PART2)
	D_CTF     = -( SQRT(1.0-AMP_FAC**2) * SIN(CHI) + 
     1               AMP_FAC * COS(CHI) )
C
	IF(BETA.GT.0.0) THEN
	  Es = CTF_Es (PI, dF, BETA, D_STAR, WAVELENGTH, Cs)
	ENDIF
	IF(Cc.GT.0.0 .OR. dE.GT.0.0) THEN
	  Et = CTF_Et (Cc, dE, VOLTS, WAVELENGTH, D_STAR)
	ENDIF
C
	CTF = D_CTF * Es * Et
C	
	RETURN
	END
 
C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE CTF_ASTIG     *****                 *
C  *                                                                          *
C  *    COMPUTE DEFOCUS VALUE FOR ASTIGMATIC DATA FOR A PARTICULAR DIRECTION  *
C  *     WITH RESPECT TO THE X,Y IMAGE PLANE.                                 *
C  *                                                                          *
C  *    NOTE: FOR NON-ASTIGMATIC IMAGES (I.E. FOCUS_MAJ = FOCUS_MIN), AT EACH *
C  *          RADIUS OF THE IMAGE FFT, THE DEFOCUS REMAINS CONSTANT.          *
C  *                                                                          *
C  *    FOCUS_MAJ = DEFOCUS VALUE (IN NMS) ALONG MAJOR AXIS.                  *
C  *    FOCUS_MIN = DEFOCUS VALUE (IN NMS) ALONG MINOR AXIS.                  *
C  *    PHI       = ANGLE (IN RADIANS) SPECIFYING THE DIRECTION OF FOCUS_MAJ  *
C  *                 RELATIVE TO THE X-AXIS OF THE DIGITIZED IMAGE.           *
C  *    PSI       = ANGLE (IN RADIANS) OF A PARTICULAR FOURIER COMPONENT      *
C  *                 WHOSE DEFOCUS VALUE IS TO BE CALCULATED.                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED  6-AUG-1999 BY T. BAKER                                 * 
C  ****************************************************************************
C
	SUBROUTINE CTF_ASTIG (FOCUS, FOCUS_MAJ, FOCUS_MIN, PSI, PHI)
C
	IMPLICIT NONE
C
	REAL*4      FOCUS, FOCUS_MAJ, FOCUS_MIN, PSI, PHI, ANG
C  ----------------------------------------------------------------------------
C
	IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
	   FOCUS = FOCUS_MAJ
	 ELSE
	   ANG   = PSI - PHI					   ! IN RADIANS
	   FOCUS = FOCUS_MAJ * COS(ANG)**2 + FOCUS_MIN * SIN(ANG)**2
	ENDIF
C	
	RETURN
	END
 
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FUNCTION CTF_Es     *****                   *
C  *                                                                          *
C  *    COMPUTE VALUE OF EXPONENTIAL DECAY ARISING FROM SPATIAL COHERENCE.    *
C  *                                                                          *
C  *    The formula used here is as described by Zemlin (Ultramicroscopy 46:  *
C  *     25-32[1992]) which is:                                               *
C  *                                                                          *
C  *       Es = exp [-Q**2]                                                   *
C  *                                                                          *
C  *                             (Cs * ALPHA**3 - dF * ALPHA)                 *
C  *       where Q = BETA * PI * (--------------------------)                 *
C  *                             (       WAVELENGTH         )                 *
C  *                                                                          *
C  *       Since D_STAR (spatial frequency) = alpha/WAVELENGTH, this          *
C  *        reduces to:                                                       *
C  *                                                                          *
C  *       Q = BETA * PI * [(Cs * WAVELENGTH**2 * D_STAR**3) - (dF * D_STAR)] *
C  *                                                                          *
C  *       where Cs is the spherical aberration coefficient (in mm)           *
C  *             dF is the defocus (in microns; positive for underfocus)      *
C  *             WAVELENGTH is the electron wavelength (in nm)                *
C  *             D_STAR is the spatial frequency (in nm-1)                    *
C  *             ALPHA is the scattering angle (in radians)                   *
C  *             BETA is the illumination aperture (in milliradians)          *
C  *                                                                          *
C  *    CALCULATIONS ARE PERFORMED WITH VARIABLES IN NM UNITS.                *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED  1-SEP-1999 BY TSB.                                     * 
C  ****************************************************************************
C
	FUNCTION CTF_Es (PI, dF, BETA, D_STAR, WAVELENGTH, Cs)
C
	IMPLICIT NONE
C
	REAL*4   BETA, PI, D_STAR, WAVELENGTH, CTF_Es, Cs, dF
	REAL*8  Q, D_CTF_Es, D_STAR_CUBED, D_BETA, 
     +                    D_dF, D_Cs, WAVE_SQ
C  ----------------------------------------------------------------------------
C
	WAVE_SQ      = WAVELENGTH * WAVELENGTH
	D_BETA       = BETA/1000.0			   ! convert to radians
	D_dF         = 1000.0 * dF			   ! convert to nm
	D_Cs         = Cs * 1000000.0			   ! convert to nm
	D_STAR_CUBED = D_STAR * D_STAR * D_STAR
	Q            = D_BETA * PI * ((D_Cs * WAVE_SQ * D_STAR_CUBED)
     1                             - (D_dF * D_STAR))
	if(abs(q) .gt. 7.0) then
	   D_CTF_Es     = 0.0
	else
	   D_CTF_Es     = EXP (-Q**2 )
	endif
	CTF_Es       = D_CTF_Es
C
	RETURN
	END
 
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FUNCTION CTF_Et     *****                   *
C  *                                                                          *
C  *    COMPUTE VALUE OF EXPONENTIAL DECAY ARISING FROM TEMPORAL COHERENCE.   *
C  *                                                                          *
C  *    The formula used here is as described by Zemlin (Ultramicroscopy 46:  *
C  *     25-32[1992]) which is:                                               *
C  *                                                                          *
C  *                [ (Cc * dE * WAVELENGTH * D_STAR**2)**2]                  *
C  *       Et = exp [-(--------------------------------)   ]                  *
C  *                [ (      E                         )   ]                  *
C  *                                                                          *
C  *       where Cc is the chromatic aberration coefficient (in mm)           *
C  *             dE is the energy spread (in electron volts)                  *
C  *             E  is the accelerating votage (in electron volts)            *
C  *             WAVELENGTH is the electron wavelength (in nm)                *
C  *             D_STAR is the spatial frequency (in nm-1)                    *
C  *                                                                          *
C  *    CALCULATIONS ARE PERFORMED WITH VARIABLES IN NM UNITS.                *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 31-AUG-1999 BY TSB.                                     * 
C  ****************************************************************************
C
	FUNCTION CTF_Et (Cc, dE, E, WAVELENGTH, D_STAR)
C
	IMPLICIT NONE
C
	REAL*4   CTF_Et, D_STAR, E, WAVELENGTH, Cc, dE
	REAL*8 D_Et, D_STAR_SQ, PART1, D_Cc, D_WAVE
C  ----------------------------------------------------------------------------
C
	D_WAVE    = WAVELENGTH
	D_Cc      = Cc * 1000000.0
	D_STAR_SQ = D_STAR * D_STAR
	PART1     = D_Cc * D_WAVE * (dE/E) * D_STAR_SQ
	if(abs(part1) .gt. 7.0) then
	   D_Et      = 0.0
	else
	   D_Et      = EXP ( -PART1*PART1 )
	endif
	CTF_Et    = D_Et
C
	RETURN
	END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE CTF_FIRSTPEAK    *****          *
C  *                                                                  *
C  *    Written by Wei Zhang.                                         *
C  ********************************************************************
C
        SUBROUTINE CTF_FIRSTPEAK ( defocus, wavelength, fft_rad,
     1                             pixel_size, IDIM, Cs, units)
C
        implicit none
C
        integer*4 idim, units
        real*4    defocus, wavelength, pixel_size, Cs, fft_rad
        double precision  fw, A, B, C, pixsize2
C
        C       = wavelength * 10.0  ! We want wavelength in Angstroms 
        A       = Cs*10000000.0            ! Want to use Cs in Angstroms
        B       = -2.0*defocus*10000.0     ! 10000. converts defocus to Angstroms              
        fw      = sqrt((-sqrt(B*B - 4.0*A*C) - B)/(2.0*A))
        if(units.eq.2) then
           pixsize2 = pixel_size * 10.0 ! Convert from nm to Angstroms
        else
           pixsize2 = pixel_size        ! Already Angstroms
        endif
Ca ORIG         fft_rad = fw*(pixel_size*idim)/ C
        fft_rad = fw*(pixsize2*idim) / C
C
        return
        end
